-----------------------------------------------------------------
-- Header
-----------------------------------------------------------------
	do
		BreadCrumbs = CreateFrame("Frame", "BreadCrumbs", UIParent); --Creates a new UI frame called "BreadCrumbs"
		
		BreadCrumbs.Name = "BreadCrumbs"
		
		BreadCrumbs.Version = GetAddOnMetadata(BreadCrumbs.Name, "Version")
		
		BreadCrumbs.LoadedStatus = {}; -- Says what stage the addon loading is at.
		BreadCrumbs.LoadedStatus["Initialized"] = false;
	end;
-----------------------------------------------------------------



-----------------------------------------------------------------
-- Libraries
-----------------------------------------------------------------
	local ZMC; -- Registering local variables outside do
	do
		BreadCrumbs.Astrolabe = DongleStub("Astrolabe-0.4-QuestHelper_ZAS")
		BreadCrumbs.AceHook = LibStub("AceHook-3.0");
		ZMC = LibStub("LibZasMsgCtr-0.2");
	end;
-----------------------------------------------------------------



----------------------------------------------------------------------------------------------------------------------
-- DEFAULTS and Variable declarations
----------------------------------------------------------------------------------------------------------------------
	local minimapParent, lastX, lastY; -- Pulling local declaration outside hider do
	do
		--------------------------------------------------------------------------
		-- Colour/Opacity defaults
		--------------------------------------------------------------------------
			--------------------------------------------------------
			-- Minimap
			--------------------------------------------------------
				-----------------------------------------
				-- Minimap Colour 1
				-----------------------------------------
					BreadCrumbs.MinimapColour1Default_Red = 1;
					BreadCrumbs.MinimapColour1Default_Green = 0;
					BreadCrumbs.MinimapColour1Default_Blue = 0;
					BreadCrumbs.MinimapColour1Default_Alpha = 0.2;
				-----------------------------------------
				
				
				
				
				-----------------------------------------
				-- Minimap Colour 2
				-----------------------------------------
					BreadCrumbs.MinimapColour2Default_Red = 1;
					BreadCrumbs.MinimapColour2Default_Green = 0;
					BreadCrumbs.MinimapColour2Default_Blue = 0;
					BreadCrumbs.MinimapColour2Default_Alpha = 0.2;
				-----------------------------------------
			--------------------------------------------------------
			
			
			
			
			
			--------------------------------------------------------
			-- HUD
			--------------------------------------------------------
				-----------------------------------------
				-- HUD Colour 1
				-----------------------------------------
					BreadCrumbs.HUDColour1Default_Red = 1;
					BreadCrumbs.HUDColour1Default_Green = 0;
					BreadCrumbs.HUDColour1Default_Blue = 0;
					BreadCrumbs.HUDColour1Default_Alpha = 0.2;
				-----------------------------------------
				
				
				
				
				-----------------------------------------
				-- HUD Colour 2
				-----------------------------------------
					BreadCrumbs.HUDColour2Default_Red = 1;
					BreadCrumbs.HUDColour2Default_Green = 0;
					BreadCrumbs.HUDColour2Default_Blue = 0;
					BreadCrumbs.HUDColour2Default_Alpha = 0.2;
				-----------------------------------------
			--------------------------------------------------------
		--------------------------------------------------------------------------
		
		minimapParent = Minimap -- Store a reference to the minimap parent
		lastX, lastY = 0,0; -- Starting values
		
		BreadCrumbs.MinimapOpacityDefault = 0.2; -- REMOVE!
		BreadCrumbs.HUDOpacityDefault = 0.2; -- REMOVE!
		
		BreadCrumbs.SpaceBetweenPointsDefault = 0.002;
		BreadCrumbs.MinimapSizeOfPoints1Default_Width = 10;
		BreadCrumbs.MinimapSizeOfPoints1Default_Hight = 10;
		BreadCrumbs.HUDSizeOfPoints1Default_Width = 30;
		BreadCrumbs.HUDSizeOfPoints1Default_Hight = 30;
		BreadCrumbs.DefaultNumPoints = 20; -- Default number of points to store (The more the longer the trail will be)
		
		BreadCrumbs.HiddenTexture = "";
		BreadCrumbs.DefaultTexture = "Interface\\AddOns\\BreadCrumbs\\Artwork\\PointBlob.tga";
	end;
----------------------------------------------------------------------------------------------------------------------



-----------------------------------------------------------------
-- KeyBinding Variables
-----------------------------------------------------------------
	do
		BINDING_HEADER_BREADCRUMBS = "BreadCrumbs";
		BINDING_NAME_ToggleAddon = "Enable/Disable BreadCrumbs";
		BINDING_NAME_ToggleBreadDrop = "Start/Stop Dropping BreadCrumbs";
	end;
-----------------------------------------------------------------


function BreadCrumbs:OnEvent(event)
	----------------------------------------------
	-- Default: If DebugTxt is set to true all of
	-- the debug msgs in THIS function will apear!
	----------------------------------------------
	local DebugTxt = false;
	-- DebugTxt = true; -- Uncomment this to debug
	----------------------------------------------
	
	ZMC:Msg(self, "BreadCrumbs:OnEvent("..tostring(event)..") self.LoadedStatus[Initialized] = "..tostring(self.LoadedStatus["Initialized"]).." arg1 = "..tostring(arg1),true,DebugTxt);
	
	if (event == "ADDON_LOADED" and self.LoadedStatus["Initialized"] == false and arg1 == "BreadCrumbs") then
		ZMC:Msg(self, "event == ADDON_LOADED and self.LoadedStatus[Initialized] == false and arg1 == BreadCrumbs",true,DebugTxt);
		
		self:Initialize();
	end;
end;

function BreadCrumbs:SetMinimapObject(minimap)
	----------------------------------------------
	-- Default: If DebugTxt is set to true all of
	-- the debug msgs in THIS function will apear!
	----------------------------------------------
	local DebugTxt = false;
	-- DebugTxt = true; -- Uncomment this to debug
	----------------------------------------------
	
	ZMC:Msg(self, "BreadCrumbs:SetMinimapObject(minimap)",true,DebugTxt);
	
	if (Minimap == minimap) then
		ZMC:Msg(self, "To Minimap",true,DebugTxt);
		
		for pointNum, waypoint in pairs(self.Points) do
			waypoint:SetParent(minimap)
			waypoint:SetWidth(BreadCrumbs_Options.MinimapSizeOfPoints1_Width); -- Sets frame's Width (will likely be changed later in the code)
			waypoint:SetHeight(BreadCrumbs_Options.MinimapSizeOfPoints1_Hight); -- Sets frame's Hight (will likely be changed later in the code)
			waypoint.texture:SetVertexColor(BreadCrumbs_Options.MinimapColour1.Red, BreadCrumbs_Options.MinimapColour1.Green, BreadCrumbs_Options.MinimapColour1.Blue, BreadCrumbs_Options.MinimapColour1.Alpha) -- Sets the colour (Red, Green, Blue, Alpha)
		end
	else
		ZMC:Msg(self, "From Minimap",true,DebugTxt);
		
		for pointNum, waypoint in pairs(self.Points) do
			waypoint:SetParent(minimap)
			waypoint:SetWidth(BreadCrumbs_Options.HUDSizeOfPoints1_Width); -- Sets frame's Width (will likely be changed later in the code)
			waypoint:SetHeight(BreadCrumbs_Options.HUDSizeOfPoints1_Hight); -- Sets frame's Hight (will likely be changed later in the code)
			waypoint.texture:SetVertexColor(BreadCrumbs_Options.HUDColour1.Red, BreadCrumbs_Options.HUDColour1.Green, BreadCrumbs_Options.HUDColour1.Blue, BreadCrumbs_Options.HUDColour1.Alpha) -- Sets the colour (Red, Green, Blue, Alpha)
		end
	end
	
	minimapParent = minimap
	self.Astrolabe:SetMinimapObject(minimap);
	self.Astrolabe.processingFrame:SetParent(minimap)
	
end

function BreadCrumbs:VarInit() -- Initialize the variables
	----------------------------------------------
	-- Default: If DebugTxt is set to true all of
	-- the debug msgs in THIS function will apear!
	----------------------------------------------
	local DebugTxt = false;
	-- DebugTxt = true; -- Uncomment this to debug
	----------------------------------------------
	
	ZMC:Msg(self, "BreadCrumbs:VarInit()",true,DebugTxt);
	
	BreadCrumbs_Options = BreadCrumbs_Options or {}; -- Initializes BreadCrumbs_Options if it doesn't already exist
	BreadCrumbs_Options.NumPoints = BreadCrumbs_Options.NumPoints or self.DefaultNumPoints; -- Initializes BreadCrumbs_Options.NumPoints with the default value if it doesn't already exist
	BreadCrumbs_Options.MinimapOpacity = BreadCrumbs_Options.MinimapOpacity or self.MinimapOpacityDefault;
	BreadCrumbs_Options.HUDOpacity = BreadCrumbs_Options.HUDOpacity or self.HUDOpacityDefault; -- REMOVE!
	
	--------------------------------------------------------------------------
	-- Colour/Opacity
	--------------------------------------------------------------------------
		--------------------------------------------------------
		-- Minimap
		--------------------------------------------------------
			-----------------------------------------
			-- Minimap Colour 1
			-----------------------------------------
				BreadCrumbs_Options.MinimapColour1 = BreadCrumbs_Options.MinimapColour1 or {};
				BreadCrumbs_Options.MinimapColour1.Red = BreadCrumbs_Options.MinimapColour1.Red or self.MinimapColour1Default_Red;
				BreadCrumbs_Options.MinimapColour1.Green = BreadCrumbs_Options.MinimapColour1.Green or self.MinimapColour1Default_Green;
				BreadCrumbs_Options.MinimapColour1.Blue = BreadCrumbs_Options.MinimapColour1.Blue or self.MinimapColour1Default_Blue;
				BreadCrumbs_Options.MinimapColour1.Alpha = BreadCrumbs_Options.MinimapColour1.Alpha or self.MinimapColour1Default_Alpha;
			-----------------------------------------
			
			
			
			
			-----------------------------------------
			-- Minimap Colour 2
			-----------------------------------------
				BreadCrumbs_Options.MinimapColour2 = BreadCrumbs_Options.MinimapColour2 or {};
				BreadCrumbs_Options.MinimapColour2.Red = BreadCrumbs_Options.MinimapColour2.Red or self.MinimapColour2Default_Red;
				BreadCrumbs_Options.MinimapColour2.Green = BreadCrumbs_Options.MinimapColour2.Green or self.MinimapColour2Default_Green;
				BreadCrumbs_Options.MinimapColour2.Blue = BreadCrumbs_Options.MinimapColour2.Blue or self.MinimapColour2Default_Blue;
				BreadCrumbs_Options.MinimapColour2.Alpha = BreadCrumbs_Options.MinimapColour2.Alpha or self.MinimapColour2Default_Alpha;
			-----------------------------------------
		--------------------------------------------------------
		
		
		
		
		
		--------------------------------------------------------
		-- HUD
		--------------------------------------------------------
			-----------------------------------------
			-- HUD Colour 1
			-----------------------------------------
				BreadCrumbs_Options.HUDColour1 = BreadCrumbs_Options.HUDColour1 or {};
				BreadCrumbs_Options.HUDColour1.Red = BreadCrumbs_Options.HUDColour1.Red or self.HUDColour1Default_Red;
				BreadCrumbs_Options.HUDColour1.Green = BreadCrumbs_Options.HUDColour1.Green or self.HUDColour1Default_Green;
				BreadCrumbs_Options.HUDColour1.Blue = BreadCrumbs_Options.HUDColour1.Blue or self.HUDColour1Default_Blue;
				BreadCrumbs_Options.HUDColour1.Alpha = BreadCrumbs_Options.HUDColour1.Alpha or self.HUDColour1Default_Alpha;
			-----------------------------------------
			
			
			
			
			-----------------------------------------
			-- HUD Colour 2
			-----------------------------------------
				BreadCrumbs_Options.HUDColour2 = BreadCrumbs_Options.HUDColour2 or {};
				BreadCrumbs_Options.HUDColour2.Red = BreadCrumbs_Options.HUDColour2.Red or self.HUDColour2Default_Red;
				BreadCrumbs_Options.HUDColour2.Green = BreadCrumbs_Options.HUDColour2.Green or self.HUDColour2Default_Green;
				BreadCrumbs_Options.HUDColour2.Blue = BreadCrumbs_Options.HUDColour2.Blue or self.HUDColour2Default_Blue;
				BreadCrumbs_Options.HUDColour2.Alpha = BreadCrumbs_Options.HUDColour2.Alpha or self.HUDColour2Default_Alpha;
			-----------------------------------------
		--------------------------------------------------------
	--------------------------------------------------------------------------
	
	BreadCrumbs_Options.SpaceBetweenPoints = BreadCrumbs_Options.SpaceBetweenPoints or self.SpaceBetweenPointsDefault;
	
	BreadCrumbs_Options.MinimapSizeOfPoints1_Width = BreadCrumbs_Options.MinimapSizeOfPoints1_Width or self.MinimapSizeOfPoints1Default_Width;
	BreadCrumbs_Options.MinimapSizeOfPoints1_Hight = BreadCrumbs_Options.MinimapSizeOfPoints1_Hight or self.MinimapSizeOfPoints1Default_Hight;
	BreadCrumbs_Options.HUDSizeOfPoints1_Width = BreadCrumbs_Options.HUDSizeOfPoints1_Width or self.HUDSizeOfPoints1Default_Width;
	BreadCrumbs_Options.HUDSizeOfPoints1_Hight = BreadCrumbs_Options.HUDSizeOfPoints1_Hight or self.HUDSizeOfPoints1Default_Hight;
	
	if (BreadCrumbs_Options.Hidden == nil) then -- Initializes BreadCrumbs_Options.Hidden with the default value of false if it doesn't already exist
		BreadCrumbs_Options.Hidden = false;
	end;
	
	if (BreadCrumbs_Options.Dropping == nil) then -- Initializes BreadCrumbs_Options.Dropping with the default value of true if it doesn't already exist
		BreadCrumbs_Options.Dropping = true;
	end;
	
	if (BreadCrumbs_Options.Enabled == nil) then -- Initializes BreadCrumbs_Options.Enabled with the default value of true if it doesn't already exist
		BreadCrumbs_Options.Enabled = true;
	end;
	
	if BreadCrumbs_Options.Hidden then
		BreadCrumbs_Options.Texture = self.HiddenTexture;
		
		ZMC:Msg(self, "WARNING: BreadCrumb trail Hidden!",false,DebugTxt,true);
	else
		BreadCrumbs_Options.Texture = self.DefaultTexture;
	end;
	
	if not(BreadCrumbs_Options.Dropping) then
		ZMC:Msg(self, "WARNING: BreadCrumb dropping disabled!",false,DebugTxt,true);
	end;
	
	if not(BreadCrumbs_Options.Enabled) then
		ZMC:Msg(self, "WARNING: Addon disabled!",false,DebugTxt,true);
	end;
	
	self.last_update = 0 -- Stores the time since the last update was performed
	
	self.LoadedStatus["AddonVariables"] = true;
end

function BreadCrumbs:Initialize() -- Initialize the addon
	----------------------------------------------
	-- Default: If DebugTxt is set to true all of
	-- the debug msgs in THIS function will apear!
	----------------------------------------------
	local DebugTxt = false;
	-- DebugTxt = true; -- Uncomment this to debug
	----------------------------------------------
	
	ZMC:Msg(self, "BreadCrumbs:Initialize()",true,DebugTxt);
	
	self:UnregisterEvent("ADDON_LOADED");
	
	self:VarInit(); -- Initialize the variables
	
	
	ZMC:Initialize(BreadCrumbs, BreadCrumbs.Name, ZMC.COLOUR_ORANGE) -- Initialize the debugging/messaging library's settings for this addon (CallingAddon, AddonName, DefaultColour1, Debug_Frame, DefaultColour2, DefaultErrorColour, DefaultMsgFrame)
	
	--- Slash Command Handler
	SLASH_BreadCrumbs1 = "/BC";
	SLASH_BreadCrumbs2 = "/BreadCrumbs";
	SlashCmdList["BreadCrumbs"] = function(msg)
		self:SlashCmdHandler(msg);
	end;
	
	if not self.UpdateFrame then
		self.UpdateFrame = CreateFrame("Frame")
		self.UpdateFrame:SetScript("OnUpdate", function(frame, elapsed) BreadCrumbs:Update(frame, elapsed); end)
	end
	
	self.OptionsPanel = LibStub("LibSimpleOptions-1.0-AlphaColourFix").AddOptionsPanel("BreadCrumbs", function(panel) self:CreateInterfaceOptions(panel) end); -- Create the OptionsPanel for BreadCrumbs
	
	if (HudMapCluster) then -- If the SexyMap HUD exists then hook it's show and hide scripts
		self.AceHook:HookScript(HudMapCluster, "OnShow", function() BreadCrumbs:HudMapCluster_OnShow() end)
		self.AceHook:HookScript(HudMapCluster, "OnHide", function() BreadCrumbs:HudMapCluster_OnHide() end)
	end;
	
	self:GeneratePoints(); -- Creates the frames that will be used as the points on the map
	
	self:TogAddon(BreadCrumbs_Options.Enabled);
	
	self.LoadedStatus["Initialized"] = true;
	
	ZMC:Msg(self, "Loaded");
end;

function BreadCrumbs:GeneratePoints() -- Creates the frames that will be used as the points on the map
	----------------------------------------------
	-- Default: If DebugTxt is set to true all of
	-- the debug msgs in THIS function will apear!
	----------------------------------------------
	local DebugTxt = false;
	-- DebugTxt = true; -- Uncomment this to debug
	----------------------------------------------
	
	ZMC:Msg(self, "BreadCrumbs:GeneratePoints()",true,DebugTxt);
	
	self.Astrolabe:RemoveAllMinimapIcons(); -- Removes all minimap icons
		
	self:WipePoints(); -- Removes all minimap icons
	
	self.Points = {}; -- Initialize the array to store the points

	local Red, Green, Blue, TextureOpacity, Width, Hight;
	if (MinimapCluster:IsShown()) then
		ZMC:Msg(self, "MinimapOpacity",true,DebugTxt);
		
		Red = BreadCrumbs_Options.MinimapColour1.Red;
		Green = BreadCrumbs_Options.MinimapColour1.Green;
		Blue = BreadCrumbs_Options.MinimapColour1.Blue;
		TextureOpacity = BreadCrumbs_Options.MinimapColour1.Alpha;
		
		Width = BreadCrumbs_Options.MinimapSizeOfPoints1_Width;
		Hight = BreadCrumbs_Options.MinimapSizeOfPoints1_Hight;
	else
		ZMC:Msg(self, "HUDOpacity",true,DebugTxt);
		
		Red = BreadCrumbs_Options.HUDColour1.Red;
		Green = BreadCrumbs_Options.HUDColour1.Green;
		Blue = BreadCrumbs_Options.HUDColour1.Blue;
		TextureOpacity = BreadCrumbs_Options.HUDColour1.Alpha;
		
		Width = BreadCrumbs_Options.HUDSizeOfPoints1_Width;
		Hight = BreadCrumbs_Options.HUDSizeOfPoints1_Hight;
	end;
	
	----------------------------------------------
	-- Generate the specified number of points
	----------------------------------------------
	for i=1, BreadCrumbs_Options.NumPoints do -- Loop for every point required
		local Point = CreateFrame("Frame", nil, minimapParent); -- Create the frame that will be the point on the minimap
		Point:SetFrameLevel(1); -- Sets the frame level (this says if its on front of or behind other frames)
		Point:SetWidth(Width); -- Sets frame's Width (will likely be changed later in the code)
		Point:SetHeight(Hight); -- Sets frame's Hight (will likely be changed later in the code)
		local texture = Point:CreateTexture(nil, "OVERLAY"); -- Creates a textrue for the frame
		Point.texture = texture; -- Attaches the texture to the frame
		texture:SetAllPoints(Point); -- ????
		texture:SetTexture(BreadCrumbs_Options.Texture); -- Sets the texture(image file) of the texture... (this could get confusing! ;-))
		-- texture:SetTexCoord(0, 1, 0, 1); -- ????
		texture:SetVertexColor(Red, Green, Blue, TextureOpacity) -- Sets the colour (Red, Green, Blue, Alpha)
		Point:Hide(); -- Hides the frame until it is needed
		self.Points[i] = Point
	end;
	----------------------------------------------
	
	-- ZMC:Msg(self, "self.Points = "..tostring(self.Points).."",true,DebugTxt);
	-- for name, value in pairs(self.Points) do
		-- ZMC:Msg(self, "name = "..tostring(name)..", value = "..tostring(value).."",true,DebugTxt);
	-- end;
end;

function BreadCrumbs:SlashCmdHandler(msg) -- Slash Command Handler
	----------------------------------------------
	-- Default: If DebugTxt is set to true all of
	-- the debug msgs in THIS function will apear!
	----------------------------------------------
	local DebugTxt = false;
	-- DebugTxt = true; -- Uncomment this to debug
	----------------------------------------------
	
	ZMC:Msg(self, "Initialize",true,DebugTxt);
	ZMC:Msg(self, "msg = '"..tostring(msg).."'",true,DebugTxt);
	
	ZMC:Msg(self, "strlower(strsub(msg,1,8) = "..tostring(strlower(strsub(msg,1,8))),true,DebugTxt);
	
	if (msg == nil or msg == "") then
		ZMC:Msg(self, "BreadCrumbs is running");
	elseif ((strlower(strsub(msg,1,7)) == "options") or (strlower(strsub(msg,1,6)) == "config")) then -- This is the macro checking in...
		ZMC:Msg(self, "Opening BreadCrumbs Options Panel");
		
		InterfaceOptionsFrame_OpenToCategory(self.OptionsPanel.name);
	elseif(strlower(strsub(msg,1,6)) == "enable") then -- This is the macro checking in...
		ZMC:Msg(self, "Enableing BreadCrumbs");
		
		self:TogAddon(true);
	elseif(strlower(strsub(msg,1,7)) == "disable") then -- This is the macro checking in...
		ZMC:Msg(self, "Disabling BreadCrumbs");
		
		self:TogAddon(false);
	else
		ZMC:Msg(self, "Message not known = '"..tostring(msg).."'",true,Debugtxt);
	end;
end;

function BreadCrumbs:Update(frame, elapsed)
	----------------------------------------------
	-- Default: If DebugTxt is set to true all of
	-- the debug msgs in THIS function will apear!
	----------------------------------------------
	local DebugTxt = false;
	-- DebugTxt = true; -- Uncomment this to debug
	----------------------------------------------
	
	ZMC:Msg(self, "BreadCrumbs:Update()",true,DebugTxt);
	
	if (not(BreadCrumbs_Options.Enabled) or (self.LoadedStatus["Initialized"] == false) or not(BreadCrumbs_Options.Dropping)) then -- Addon is disabled so exit
		return;
	end;
	
	-- update our zone info
	local curZone = GetRealZoneText()
	if (not curZone or curZone == "") then 
		curZone = nil
		return
	end
	
	self.last_update = self.last_update + elapsed -- Calculates how long since it last updated
	ZMC:Msg(self, "self.UpdateFrame - OnUpdate",true,DebugTxt);
	ZMC:Msg(self, "elapsed = "..tostring(elapsed),true,DebugTxt);
	ZMC:Msg(self, "last_update = "..tostring(self.last_update),true,DebugTxt);
	
	if self.last_update > 0.1 then -- Ensures that it doesn't update to often
		ZMC:Msg(self, "BreadCrumbs:Update() - last_update > 0.5",true,DebugTxt);
		
		self.last_update = 0
		
		local zoneIndex = GetCurrentMapZone();
		local continentIndex = GetCurrentMapContinent();
		local SpaceBetweenPoints = BreadCrumbs_Options.SpaceBetweenPoints;
		ZMC:Msg(self, "zoneIndex = "..tostring(zoneIndex).."",true,DebugTxt);
		ZMC:Msg(self, "continentIndex = "..tostring(continentIndex).."",true,DebugTxt);
		
		-----------------------------------------------------------------
		-- Store the players current position on the map in the next slot
		----------------------------------------------------------------
			-- Get current player position
			local posX, posY = GetPlayerMapPosition("player");
			
			ZMC:Msg(self, "posX = "..tostring(posX),true,DebugTxt);
			ZMC:Msg(self, ", posY = "..tostring(posY),true,DebugTxt);
			ZMC:Msg(self, " curZone = "..tostring(curZone).."",true,DebugTxt);
			ZMC:Msg(self, "",true,DebugTxt);
			
			local PrevMovementHistorySlot = 1;
			if ((self.NextMovementHistorySlot - 1) < 1) then
				PrevMovementHistorySlot = BreadCrumbs_Options.NumPoints;
			else 
				PrevMovementHistorySlot = self.NextMovementHistorySlot - 1;
			end;
			
			local xDif = (posX - self.MovementHistory[PrevMovementHistorySlot]["x"])
			local yDif = (posY - self.MovementHistory[PrevMovementHistorySlot]["y"])
			
			if (((posX == 0) or (posY == 0) or GetCurrentMapZone() == 0)) then -- If position is 0, the player changed the worldmap to another zone, just keep the old values
				ZMC:Msg(self, "posX = "..tostring(posX).." posY = "..tostring(posY).." GetCurrentMapZone() = "..tostring(GetCurrentMapZone()).."",true,DebugTxt);
				ZMC:Msg(self, "The player changed the worldmap to another zone, don't do anything",true,DebugTxt);
			else -- Map is not
				ZMC:Msg(self, "SpaceBetweenPoints = "..tostring(SpaceBetweenPoints).."",true,DebugTxt);
				if (((xDif > SpaceBetweenPoints) or (xDif < -SpaceBetweenPoints) or (yDif > SpaceBetweenPoints) or (yDif < -SpaceBetweenPoints)) and not(posX == 0 and posY == 0)) then
					ZMC:Msg(self, "self.MovementHistory[PrevMovementHistorySlot][x] = "..tostring(self.MovementHistory[PrevMovementHistorySlot]["x"]).."self.MovementHistory[PrevMovementHistorySlot][y] = "..tostring(self.MovementHistory[PrevMovementHistorySlot]["y"]),true,DebugTxt);
					ZMC:Msg(self, "Diference IS great enough!",true,DebugTxt);
					ZMC:Msg(self, "self.MovementHistorySlot = "..tostring(self.NextMovementHistorySlot),true,DebugTxt);
					
					self.MovementHistory[self.NextMovementHistorySlot]["x"] = posX;
					self.MovementHistory[self.NextMovementHistorySlot]["y"] = posY;
					self.MovementHistory[self.NextMovementHistorySlot]["zone"] = curZone;
					self.MovementHistory[self.NextMovementHistorySlot]["continentIndex"] = continentIndex;
					self.MovementHistory[self.NextMovementHistorySlot]["zoneIndex"] = zoneIndex;
					
					if (self.NextMovementHistorySlot + 1) < (BreadCrumbs_Options.NumPoints + 1) then
						self.NextMovementHistorySlot = self.NextMovementHistorySlot + 1;
					else
						self.NextMovementHistorySlot = 1;
					end;
					
					ZMC:Msg(self, "self.NextMovementHistorySlot = "..tostring(self.NextMovementHistorySlot).."",true,DebugTxt);
					local result = self.Astrolabe:RemoveIconFromMinimap(self.Points[self.NextMovementHistorySlot])
					if not(result == 0) then -- There was an error!
						ZMC:Msg(self, "There was an error trying to remove an icon from the minimap. Icon # = "..tostring(self.NextMovementHistorySlot).."",true,DebugTxt);
					end;
					
					local result = self.Astrolabe:PlaceIconOnMinimap(self.Points[self.NextMovementHistorySlot], continentIndex, zoneIndex, posX, posY);
					if not(result == 0) then -- There was an error!
						ZMC:Msg(self, "There was an error trying to add icon to minimap.  Icon # = "..tostring(self.NextMovementHistorySlot).."",true,DebugTxt);
					end;
					
					
					ZMC:Msg(self, "self.NextMovementHistorySlot = "..tostring(self.NextMovementHistorySlot),true,DebugTxt);
				else
					ZMC:Msg(self, "Diference not great enough! xDif = "..tostring(xDif).." yDif = "..tostring(yDif).."",true,DebugTxt);
				end;		
			end;
			
			ZMC:Msg(self, "xDif = "..tostring(xDif),true,DebugTxt);
			ZMC:Msg(self, "yDif = "..tostring(yDif),true,DebugTxt);
		-----------------------------------------------------------------
	end;
end;

function BreadCrumbs:HudMapCluster_OnShow()
	----------------------------------------------
	-- Default: If DebugTxt is set to true all of
	-- the debug msgs in THIS function will apear!
	----------------------------------------------
	local DebugTxt = false;
	-- DebugTxt = true; -- Uncomment this to debug
	----------------------------------------------
	
	ZMC:Msg(self, "BreadCrumbs:HudMapCluster_OnShow()",true,DebugTxt);
	
	if (not(BreadCrumbs_Options.Enabled)) then -- Addon is disabled so exit
		return;
	end;
	
	BreadCrumbs:SetMinimapObject(HudMapCluster)
end;

function BreadCrumbs:HudMapCluster_OnHide()
	----------------------------------------------
	-- Default: If DebugTxt is set to true all of
	-- the debug msgs in THIS function will apear!
	----------------------------------------------
	local DebugTxt = false;
	-- DebugTxt = true; -- Uncomment this to debug
	----------------------------------------------
	
	ZMC:Msg(self, "BreadCrumbs:HudMapCluster_OnHide()",true,DebugTxt);
	
	if (not(BreadCrumbs_Options.Enabled)) then -- Addon is disabled so exit
		return;
	end;
	
	
	BreadCrumbs:SetMinimapObject(Minimap)
end;

function BreadCrumbs:CreateInterfaceOptions(panel) -- Adds an options frame to the Blizzard Interface Frame for this addon
	----------------------------------------------
	-- Default: If DebugTxt is set to true all of
	-- the debug msgs in THIS function will apear!
	----------------------------------------------
	local DebugTxt = false;
	-- DebugTxt = true; -- Uncomment this to debug
	----------------------------------------------
	
	ZMC:Msg(self, "BreadCrumbs:CreateInterfaceOptions("..tostring(panel)..")",true,DebugTxt);
	
	
	
	panel.title, panel.subtitle = panel:MakeTitleTextAndSubText("General Options", "These options allow you to configure various aspects of BreadCrumbs.");
	
	
	
	
	--------------------------------------------------------------------------------------------
	-- Creates and Sets Up the Addon Enable Toggle
	--------------------------------------------------------------------------------------------
		panel.Enable = panel:MakeToggle(
										"name", "Enabled",
										"description", "Turns BreadCrumbs On/Off. If you don't plan to use it for a long time or just don't want to use it on this toon your better disabling it via the 'Addon' menu on the toon selection screen to save memory.",
										"default", true,
										"getFunc", function() return BreadCrumbs_Options.Enabled end,
										"setFunc", function(value) self:TogAddon(value) end
									   );
		panel.Enable:SetPoint("TOPLEFT", panel.subtitle, "BOTTOMLEFT", 0, -10);
	--------------------------------------------------------------------------------------------
	
	
	--------------------------------------------------------------------------------------------
	-- Creates and Sets Up the First Point's Colour/Opacity for the Minimap
	--------------------------------------------------------------------------------------------
		panel.MinimapColour1 = panel:MakeColorPicker(
				"name", "Colour/Opacity (Minimap)",
				"description", "This sets the Colour & Opacity level of the points that make up the BreadCrumbs when on the Minimap.\n\n"..self.ZMC_DefaultColour1.."|r",
				"hasAlpha", true,
				"defaultR", self.MinimapColour1Default_Red,
				"defaultG", self.MinimapColour1Default_Green,
				"defaultB", self.MinimapColour1Default_Blue,
				"defaultA", self.MinimapColour1Default_Alpha,
				"currentR", BreadCrumbs_Options.MinimapColour1.Red,
				"currentG", BreadCrumbs_Options.MinimapColour1.Green,
				"currentB", BreadCrumbs_Options.MinimapColour1.Blue,
				"currentA", BreadCrumbs_Options.MinimapColour1.Alpha,
				"setFunc", function(r, g, b, a)
					BreadCrumbs_Options.MinimapColour1.Red = r;
					BreadCrumbs_Options.MinimapColour1.Green = g;
					BreadCrumbs_Options.MinimapColour1.Blue = b;
					BreadCrumbs_Options.MinimapColour1.Alpha = a;
					
					self:UpdatePoints();
				end
			)
		panel.MinimapColour1:SetPoint("TOPLEFT", panel.Enable, "BOTTOMLEFT", 0, -10);
	--------------------------------------------------------------------------------------------
	
	
	--------------------------------------------------------------------------------------------
	-- Creates and Sets Up the First Point's Colour/Opacity for the HUD
	--------------------------------------------------------------------------------------------
		local HUDOpacity_Name = "Colour/Opacity (SexyMapHUD)";
		if (not(HudMapCluster)) then -- The SexyMap HUD doesn't exist so gray out options
			HUDOpacity_Name = ZMC.COLOUR_MEDGRAY..HUDOpacity_Name..ZMC.COLOUR_CLOSE;
		end;
		
		panel.HUDColour1 = panel:MakeColorPicker(
				"name", HUDOpacity_Name,
				"description", "This sets the Colour & Opacity level of the points that make up the BreadCrumbs when on the SexyMap HUD.\n\n"..self.ZMC_DefaultColour1.."NOTE: Setting this to 100% visable will only make it 100% visable if the SexyMap HUD is also 100% visable.|r",
				"hasAlpha", true,
				"defaultR", self.MinimapColour1Default_Red,
				"defaultG", self.MinimapColour1Default_Green,
				"defaultB", self.MinimapColour1Default_Blue,
				"defaultA", self.MinimapColour1Default_Alpha,
				"currentR", BreadCrumbs_Options.HUDColour1.Red,
				"currentG", BreadCrumbs_Options.HUDColour1.Green,
				"currentB", BreadCrumbs_Options.HUDColour1.Blue,
				"currentA", BreadCrumbs_Options.HUDColour1.Alpha,
				"setFunc", function(r, g, b, a)
					BreadCrumbs_Options.HUDColour1.Red = r;
					BreadCrumbs_Options.HUDColour1.Green = g;
					BreadCrumbs_Options.HUDColour1.Blue = b;
					BreadCrumbs_Options.HUDColour1.Alpha = a;
					
					self:UpdatePoints();
				end
			)
		panel.HUDColour1:SetPoint("TOPLEFT", panel.MinimapColour1, "BOTTOMLEFT", 0, -10);
		
		if (not(HudMapCluster)) then -- The SexyMap HUD doesn't exist so disable options
			panel.HUDColour1:Disable();
		end;
	--------------------------------------------------------------------------------------------
	
	
	--------------------------------------------------------------------------------------------
	-- Creates and Sets Up the SpaceBetweenPoints slider
	--------------------------------------------------------------------------------------------
		panel.SpaceBetweenPoints = panel:MakeSlider(
				"name", "Space Between Points",
				"description", "This slider specifies how close together the points are that make up the line. Closer together gives a better looking line but makes it shorter.\n\n"..self.ZMC_DefaultColour1.."NOTE: This will also affect the length of the line so you may need to also adjust the number of points used for the line.\n\nAlso this will only affect new BreadCrumbs and not your existing trail.|r",
				"minText", "Really Close",
				"maxText", "Miles Apart!",
				"minValue", 0.0001,
				"maxValue", 0.0091,
				"step", 0.001,
				"default", 0.002,
				"current", BreadCrumbs_Options.SpaceBetweenPoints,
				"setFunc", function(value) BreadCrumbs_Options.SpaceBetweenPoints = value; end
			)
		panel.SpaceBetweenPoints:SetPoint("TOPLEFT", panel.HUDColour1, "BOTTOMLEFT", 0, -30);
	--------------------------------------------------------------------------------------------
	
	
	--------------------------------------------------------------------------------------------
	-- Creates and Sets Up the NumberOfPoints slider
	--------------------------------------------------------------------------------------------
		panel.NumberOfPoints = panel:MakeSlider(
				"name", "Number of Points",
				"description", "This slider specifies how many points are used to create the line and therefore how long it will be.\n\n"..self.ZMC_DefaultColour1.."NOTE: Use this and the spacing one to crate the line you want.\n\nALSO this will reset your BreadCrumbs(remove your trail)! So ensure you make a note of which way you have come from.|r\n\n"..self.ZMC_DefaultErrorColour.."WARNING! Setting this higher MAY slow the game down!|r",
				"minText", "1",
				"maxText", "100",
				"minValue", 1,
				"maxValue", 100,
				"step", 1,
				"default", 20,
				"current", BreadCrumbs_Options.NumPoints,
				"setFunc", function(value)
					if (BreadCrumbs_Options.NumPoints == value) then -- If no change then don't do anything!
						ZMC:Msg(self, "BreadCrumbs_Options.NumPoints == value",true,DebugTxt);
						return;
					end;
					
					BreadCrumbs_Options.NumPoints = value; -- Set the number of points to the ones specified
					
					self:GeneratePoints(); -- Recreate the new number of points
					
					if (HudMapCluster) then
						ZMC:Msg(self, "HudMapCluster Exists!",true,DebugTxt);
						
						if (HudMapCluster:IsShown()) then
							ZMC:Msg(self, "and HudMapCluster is Shown!",true,DebugTxt);
							
							HudMapCluster:Hide(); -- Hide the SexyMap's HUD
							HudMapCluster:Show(); -- Now Show the SexyMap's HUD again to reset everything.
						-- else
							-- ZMC:Msg(self, "and HudMapCluster is NOT Shown!",true,DebugTxt);
						end;
					-- else
						-- ZMC:Msg(self, "HudMapCluster Does NOT Exist!",true,DebugTxt);
					end;
				end,
				"currentTextFunc", function(value) return tostring(value) end
			)
		panel.NumberOfPoints:SetPoint("TOPLEFT", panel.SpaceBetweenPoints, "BOTTOMLEFT", 0, -30);
	--------------------------------------------------------------------------------------------
	
	
	--------------------------------------------------------------------------------------------
	-- Creates and Sets Up the MinimapSizeOfPoints1 slider
	--------------------------------------------------------------------------------------------
		panel.MinimapSizeOfPoints1 = panel:MakeSlider(
				"name", "Size(Minimap)",
				"description", "This slider specifies how big the points on the minimap are.|r",
				"minText", "Tiny",
				"maxText", "HUGE!",
				"minValue", 1,
				"maxValue", 50,
				"step", 1,
				"default", BreadCrumbs.MinimapSizeOfPoints1Default_Width,
				"current", BreadCrumbs_Options.MinimapSizeOfPoints1_Width, -- Only uses the width as currently the width and hight are the same size... but storing both encase I change my mind! ;-)
				"setFunc", function(value)
					BreadCrumbs_Options.MinimapSizeOfPoints1_Width = value;
					BreadCrumbs_Options.MinimapSizeOfPoints1_Hight = value;
					
					self:UpdatePoints();
				end
			)
		panel.MinimapSizeOfPoints1:SetPoint("TOPLEFT", panel.NumberOfPoints, "BOTTOMLEFT", 0, -30);
	--------------------------------------------------------------------------------------------
	
	
	--------------------------------------------------------------------------------------------
	-- Creates and Sets Up the HUDSizeOfPoints1 slider
	--------------------------------------------------------------------------------------------
		local HUDSizeOfPoints1_Name = "Size(SexyMapHUD)";
		if (not(HudMapCluster)) then -- The SexyMap HUD doesn't exist so gray out options
			HUDSizeOfPoints1_Name = ZMC.COLOUR_MEDGRAY..HUDSizeOfPoints1_Name..ZMC.COLOUR_CLOSE;
		end;
		
		panel.HUDSizeOfPoints1 = panel:MakeSlider(
				"name", HUDSizeOfPoints1_Name,
				"description", "This slider specifies how big the points on the SexyMapHUD.|r",
				"minText", "Tiny",
				"maxText", "HUGE!",
				"minValue", 1,
				"maxValue", 50,
				"step", 1,
				"default", BreadCrumbs.HUDSizeOfPoints1Default_Width,
				"current", BreadCrumbs_Options.HUDSizeOfPoints1_Width, -- Only uses the width as currently the width and hight are the same size... but storing both encase I change my mind! ;-)
				"setFunc", function(value)
					BreadCrumbs_Options.HUDSizeOfPoints1_Width = value;
					BreadCrumbs_Options.HUDSizeOfPoints1_Hight = value;
					
					self:UpdatePoints();
				end
			)
		panel.HUDSizeOfPoints1:SetPoint("TOPLEFT", panel.MinimapSizeOfPoints1, "BOTTOMLEFT", 0, -30);
		
		if (not(HudMapCluster)) then -- The SexyMap HUD doesn't exist so disable options
			panel.HUDSizeOfPoints1:Disable();
		end;
	--------------------------------------------------------------------------------------------
end;

function BreadCrumbs:UpdatePoints() -- Updates the Colour/Opacity and Size of all of the points
	----------------------------------------------
	-- Default: If DebugTxt is set to true all of
	-- the debug msgs in THIS function will apear!
	----------------------------------------------
	local DebugTxt = false;
	-- DebugTxt = true; -- Uncomment this to debug
	----------------------------------------------
	
	ZMC:Msg(self, "BreadCrumbs:UpdatePoints()",true,DebugTxt);
	
	local Red, Green, Blue, TextureOpacity;
	if (MinimapCluster:IsShown()) then
		ZMC:Msg(self, "MinimapOpacity",true,DebugTxt);
		
		Red = BreadCrumbs_Options.MinimapColour1.Red;
		Green = BreadCrumbs_Options.MinimapColour1.Green;
		Blue = BreadCrumbs_Options.MinimapColour1.Blue;
		TextureOpacity = BreadCrumbs_Options.MinimapColour1.Alpha;
		
		Width = BreadCrumbs_Options.MinimapSizeOfPoints1_Width;
		Hight = BreadCrumbs_Options.MinimapSizeOfPoints1_Hight;
	else
		ZMC:Msg(self, "HUDOpacity",true,DebugTxt);
		
		Red = BreadCrumbs_Options.HUDColour1.Red;
		Green = BreadCrumbs_Options.HUDColour1.Green;
		Blue = BreadCrumbs_Options.HUDColour1.Blue;
		TextureOpacity = BreadCrumbs_Options.HUDColour1.Alpha;
		
		Width = BreadCrumbs_Options.HUDSizeOfPoints1_Width;
		Hight = BreadCrumbs_Options.HUDSizeOfPoints1_Hight;
	end;
	
	ZMC:Msg(self, "TextureOpacity = "..tostring(TextureOpacity).."",true,DebugTxt);
	ZMC:Msg(self, "Red = "..tostring(Red)..", Green = "..tostring(Green)..", Blue = "..tostring(Blue)..", TextureOpacity = "..tostring(TextureOpacity).."",true,DebugTxt);
	for name, value in pairs(self.Points) do
		value:SetWidth(Width); -- Sets frame's Width (will likely be changed later in the code)
		value:SetHeight(Hight); -- Sets frame's Hight (will likely be changed later in the code)
		value.texture:SetVertexColor(Red, Green, Blue, TextureOpacity) -- Sets the colour (Red, Green, Blue, Alpha)
	end;
end;

function BreadCrumbs:TogAddon(value) -- Enables or Disables BreadCrumbs
	----------------------------------------------
	-- Default: If DebugTxt is set to true all of
	-- the debug msgs in THIS function will apear!
	----------------------------------------------
	local DebugTxt = false;
	-- DebugTxt = true; -- Uncomment this to debug
	----------------------------------------------
	
	ZMC:Msg(self, "BreadCrumbs:TogAddon(value = "..tostring(value)..")",true,DebugTxt);
	
	if (value == nil) then -- No value has been passed so just invert the current one
		if BreadCrumbs_Options.Enabled == true then
			BreadCrumbs_Options.Enabled = false;
			
			if (self.OptionsPanel.Enable) then
				self.OptionsPanel.Enable:SetChecked(false); -- Make sure the enable tog in the config window is correctly selected
			end;
			
			self:WipePoints(); -- Removes all minimap icons
			
			ZMC:Msg(self, "Disabled.",false,DebugTxt,true);
		else
			BreadCrumbs_Options.Enabled = true;
			
			if (self.OptionsPanel.Enable) then
				self.OptionsPanel.Enable:SetChecked(true); -- Make sure the enable tog in the config window is correctly selected
			end;
			
			self:WipePoints(); -- Removes all minimap icons
			
			ZMC:Msg(self, "Enabled.",false,DebugTxt,true);
		end;
	elseif (value == true) then -- True passed so enable addon
		if (BreadCrumbs_Options.Enabled == false) then
			self:WipePoints(); -- Removes all minimap icons
			
			ZMC:Msg(self, "Enabled.",false,DebugTxt,true);
		end;
		
		BreadCrumbs_Options.Enabled = true;
		
		if (self.OptionsPanel.Enable) then
			self.OptionsPanel.Enable:SetChecked(true); -- Make sure the enable tog in the config window is correctly selected
		end;
	elseif (value == false) then -- False passed so enable addon
		BreadCrumbs_Options.Enabled = false;
		
		if (self.OptionsPanel.Enable) then
			self.OptionsPanel.Enable:SetChecked(false); -- Make sure the enable tog in the config window is correctly selected
		end;
		
		self:WipePoints(); -- Removes all minimap icons
		
		ZMC:Msg(self, "Disabled.",false,DebugTxt,true);
	else
		ZMC:Msg(self, "ERROR value not specified",true,DebugTxt,true);
	end;
end;

function BreadCrumbs:WipePoints() -- Removes all minimap icons
	----------------------------------------------
	-- Default: If DebugTxt is set to true all of
	-- the debug msgs in THIS function will apear!
	----------------------------------------------
	local DebugTxt = false;
	-- DebugTxt = true; -- Uncomment this to debug
	----------------------------------------------
	
	ZMC:Msg(self, "BreadCrumbs:WipePoints()",true,DebugTxt);
	
	self.NextMovementHistorySlot = 1 -- Stores the next slot number for the position
	
	self.MovementHistory = {};
	for i=1, BreadCrumbs_Options.NumPoints do
		self.MovementHistory[i] = {}
		self.MovementHistory[i]["x"] = 0
		self.MovementHistory[i]["y"] = 0
	end;
	
	self.Astrolabe:RemoveAllMinimapIcons(); -- Removes all minimap icons
end;

function BreadCrumbs:TogBreadDrop(value) -- Starts/Stops dropping a trail. (This won't delete or hide the trail)
	----------------------------------------------
	-- Default: If DebugTxt is set to true all of
	-- the debug msgs in THIS function will apear!
	----------------------------------------------
	local DebugTxt = false;
	-- DebugTxt = true; -- Uncomment this to debug
	----------------------------------------------
	
	ZMC:Msg(self, "BreadCrumbs:TogBreadDrop(value = "..tostring(value)..")",true,DebugTxt);
	
	if (value == nil) then -- No value given so just toggle it to the opposite of it's current state
		if (BreadCrumbs_Options.Dropping) then
			BreadCrumbs_Options.Dropping = false;
			
			ZMC:Msg(self, "Stopped dropping BreadCrumbs",false,DebugTxt,true);
		else
			BreadCrumbs_Options.Dropping = true;
			
			ZMC:Msg(self, "Started dropping BreadCrumbs",false,DebugTxt,true);
		end;
	elseif (value) then -- Value specifed as true so turn it on
		BreadCrumbs_Options.Dropping = true;
		
		ZMC:Msg(self, "Started dropping BreadCrumbs",false,DebugTxt,true);
	else -- Value specified and it's not true so assume it's false and turn it off
		BreadCrumbs_Options.Dropping = false;
		
		ZMC:Msg(self, "Stopped dropping BreadCrumbs",false,DebugTxt,true);
	end;
end;

function BreadCrumbs:TogHideBread(value) -- Hides/unhides the trail. (This won't delete the trail and it will still be making one in the background unless it's stopped)
	----------------------------------------------
	-- Default: If DebugTxt is set to true all of
	-- the debug msgs in THIS function will apear!
	----------------------------------------------
	local DebugTxt = false;
	-- DebugTxt = true; -- Uncomment this to debug
	----------------------------------------------
	
	ZMC:Msg(self, "BreadCrumbs:TogHideBread(value = "..tostring(value)..")",true,DebugTxt);
	
	if (value == nil) then -- No value given so just toggle it to the opposite of it's current state
		if (BreadCrumbs_Options.Hidden) then
			BreadCrumbs_Options.Hidden = false;
			
			for name, value in pairs(self.Points) do
				value.texture:SetTexture(BreadCrumbs_Options.Texture)
			end;
			
			ZMC:Msg(self, "Unhidden",false,DebugTxt,true);
		else
			BreadCrumbs_Options.Hidden = true;
			
			for name, value in pairs(self.Points) do
				value.texture:SetTexture(BreadCrumbs.HiddenTexture)
			end;
			
			ZMC:Msg(self, "Hidden",false,DebugTxt,true);
		end;
	elseif (value) then -- Value specifed as true so turn it on
		if (BreadCrumbs_Options.Hidden == false) then -- Only unhide it if it's hidden... (lets you call this to unhide it knowing it won't do anything if not required)
			BreadCrumbs_Options.Hidden = true;
			
			for name, value in pairs(self.Points) do
				value.texture:SetTexture(BreadCrumbs.HiddenTexture)
			end;
			
			ZMC:Msg(self, "Hidden",false,DebugTxt,true);
		end;
	else -- Value specified and it's not true so assume it's false and turn it off
		if (BreadCrumbs_Options.Hidden == true) then -- Only hide it if it's visable... (lets you call this to hide it knowing it won't do anything if not required)
			BreadCrumbs_Options.Hidden = false;
			
			for name, value in pairs(self.Points) do
				value.texture:SetTexture(BreadCrumbs_Options.Texture)
			end;
			
			ZMC:Msg(self, "Unhidden",false,DebugTxt,true);
		end;
	end;
end;

BreadCrumbs:RegisterEvent("ADDON_LOADED"); --Catch when this addon has finished loading
BreadCrumbs:SetScript("OnEvent", BreadCrumbs.OnEvent);